#!/usr/bin/python
# Copyright (C) 2013 Nicira, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Limitations:
#     - Doesn't support multicast other than "unknown-dst"


# Modifications to add support from Broadcom switches based on OpEN API
# XXX Refactor in order to better support other platforms. 

class DebugException(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)

'''
Example usage: sudo python vtepd.py --log-file myapp.log br-int
'''

import argparse
import re
import subprocess
import sys
import signal
import os
import time

sys.path=['/etc/vtepd'] + sys.path

import time
import types
import vtepd_config

try:
    if vtepd_config.local_interface:
       pass
except:
    vtepd_config.local_interface = "eth1"

isBroadcomSwitch = False
open = None

try:
    import OpEN_py as OpEN
    from OpENUtil import *
    isBroadcomSwitch = True
except:
    pass

import ovs.dirs
import ovs.util
import ovs.daemon
import ovs.unixctl.server
import ovs.vlog
import socket
import fcntl
import struct

VERSION = "0.99"

root_prefix = ""

__pychecker__ = 'no-reuseattr'  # Remove in pychecker >= 0.8.19.
vlog = ovs.vlog.Vlog("ovs-vtep")
exiting = False

Tunnel_Ip = ""
Lswitches = {}
Bindings = {}
ls_count = 0
tun_id = 0

def receive_signal(signum, stack):
    global exiting
    vlog.info("received signal %d" % (signum))
    exiting = True
    unixctl.close()

def get_ip_address(ifname):
    '''
    Given an interface (e.g., "eth0") return it's IPV4 address.
    '''

    if isBroadcomSwitch:
        return vtepd_config.host_ip

    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    return socket.inet_ntoa(fcntl.ioctl(
        s.fileno(),
        0x8915,  # SIOCGIFADDR
        struct.pack('256s', ifname[:15])
    )[20:24])


def call_prog(prog, args_list):
    if isBroadcomSwitch:
        db = "--db=unix:/usr/local/var/run/openvswitch/db.sock"
    else:
        db = "--db=unix:/var/run/openvswitch/db.sock"

    if prog == "ovs-ofctl":
        cmd = [prog, "-vconsole:off",  "--timeout=2"] + args_list
    else:
        cmd = [prog, "-vconsole:off",  "--timeout=2", db] + args_list
    output = ""
    try:
      output = subprocess.Popen(cmd, stdout=subprocess.PIPE).communicate()
      if len(output) == 0 or output[0] == None:
        output = ""
      else:
        output = output[0].strip()
    except:
      pass
    return output

def ovs_vsctl(args):
    return call_prog("ovs-vsctl", args.split())

def ovs_ofctl(args):
    return call_prog("ovs-ofctl", args.split())

def vtep_ctl(args):
    return call_prog("vtep-ctl", args.split())


def unixctl_exit(conn, unused_argv, unused_aux):
    global exiting
    exiting = True
    conn.reply(None)


class Logical_Switch(object):
    def __init__(self, ls_name):
        global ls_count
        self.name = ls_name
        ls_count += 1
        self.short_name = "br-int"
        self.ovs_tunnel_bridge = "br-tun"
        vlog.info("creating lswitch %s (%s)" % (self.name, self.short_name))
        self.ports = {}
        self.tunnels = {}
        self.local_macs = set()
        self.remote_macs = {}
        self.unknown_dsts = set()
        self.tunnel_key = 0
        self.setup_ls()

    def __del__(self):
        vlog.info("destroying lswitch %s" % self.name)

    def cleanup(self):
        self.del_all_tunnels()
        if isBroadcomSwitch:
            self.del_tenant()

        bindings = self.ports.keys()
        for x in bindings:
            self.del_lbinding(x)

    def get_tunnels(self):
        return self.tunnels

    def setup_ls(self):
        column = vtep_ctl("--columns=tunnel_key find logical_switch "
                              "name=%s" % self.name)
        tunnel_key = column.partition(":")[2].strip()
        if (tunnel_key and type(eval(tunnel_key)) == types.IntType):
            self.tunnel_key = tunnel_key
            vlog.info("using tunnel key %s in %s"
                      % (self.tunnel_key, self.name))
        else:
            self.tunnel_key = 0
            vlog.warn("invalid tunnel key for %s, using 0" % self.name)

        vtep_ctl("clear-local-macs %s" % self.name)
        vtep_ctl("add-mcast-local %s unknown-dst %s" % (self.name, Tunnel_Ip))

    def update_flood(self):
        return
        if isBroadcomSwitch:
            return

        flood_ports = self.ports.values()

        # Traffic flowing from one 'unknown-dst' should not be flooded to
        # port belonging to another 'unknown-dst'.
        for tunnel in self.unknown_dsts:
            port_no = self.tunnels[tunnel][0]
            ovs_ofctl("add-flow %s table=1,priority=1,in_port=%s,action=%s"
                        % (self.short_name, port_no, ",".join(flood_ports)))

        # Traffic coming from a VTEP physical port should only be flooded to
        # one 'unknown-dst' and to all other physical ports that belong to
        # that
        # VTEP device and this logical switch.
        for tunnel in self.unknown_dsts:
            port_no = self.tunnels[tunnel][0]
            flood_ports.append(port_no)
            break

        ovs_ofctl("add-flow %s table=1,priority=0,action=%s"
                  % (self.short_name, ",".join(flood_ports)))

    def add_lbinding(self, lbinding):
        if isBroadcomSwitch:
            self.add_lbinding_broadcom(lbinding)
        else:
            self.add_lbinding_ovs(lbinding)

    def add_lbinding_ovs(self, lbinding):
        vlog.info("adding %s binding to %s" % (lbinding, self.name))
        port_no = ovs_vsctl("get Interface %s ofport" % lbinding)
        self.ports[lbinding] = port_no

        self.update_flood()

    def add_lbinding_broadcom(self, lbinding):

        '''
        see if we have a tenant created for the ls. if not, create it.
        see if we have the vlan created. if not, create it.
        see if the port is bound to the vlan. If not, bind it.
        '''

        try:
            vlan, pp_name = lbinding.split("-")
        except:
            vlan, pp_name0, pp_name1 = lbinding.split("-")
            pp_name = pp_name0 

        port = int(pp_name)

        open.create_vlan(vlan)

        open.create_tenant(self.tunnel_key, 
                           OpEN.OPEN_L2OL3_TUNNEL_TYPE_VXLAN, 
                           vlan)

        open.add_port_to_vlan(vlan, port)
        open.set_port_pvid(port, vlan)

        self.ports[lbinding] = port
        self.update_flood()

    def del_lbinding(self, lbinding):
        if isBroadcomSwitch:
            self.del_lbinding_broadcom(lbinding)
        else:
            self.del_lbinding_ovs(lbinding)

    def del_binding_ovs(self, lbinding):
        vlog.info("removing %s binding from %s" % (lbinding, self.name))
        port_no = self.ports[lbinding]
        ovs_ofctl("del-flows %s in_port=%s" % (self.short_name, port_no));
        del self.ports[lbinding]
        self.update_flood()

    def del_lbinding_broadcom(self, lbinding):
        try:
            vlan, pp_name = lbinding.split("-")
        except:
            vlan, pp_name0, pp_name1 = lbinding.split("-")
            pp_name = pp_name0 
    
        port = int(pp_name)

        vlog.info("removing %s binding from %s" % (lbinding, self.name))

        open.remove_port_from_vlan(vlan, port)

        del self.ports[lbinding]
        self.update_flood()

    def add_tunnel(self, tunnel):
        if isBroadcomSwitch == True:
            self.add_tunnel_broadcom(tunnel)
        else:
            self.add_tunnel_ovs(tunnel)

    def add_tunnel_ovs(self, tunnel): 
        global tun_id
        vlog.info("adding tunnel %s" % tunnel)
        encap, ip = tunnel.split("/")

        if encap != "vxlan_over_ipv4":
            vlog.warn("unsupported tunnel format %s" % encap)
            return

        tun_id += 1
        tun_name = "vx" + str(tun_id)


        ovs_vsctl("add-port %s %s -- set Interface %s type=vxlan "
                  "options:in_key=flow options:out_key=flow options:local_ip=%s options:remote_ip=%s"
                  % (self.ovs_tunnel_bridge, tun_name, tun_name, get_ip_address(vtepd_config.local_interface), ip))

        for i in range(10):
            port_no = ovs_vsctl("get Interface %s ofport" % tun_name)
            if port_no != "-1":
                break
            elif i == 9:
                vlog.warn("couldn't create tunnel %s" % tunnel)
                ovs_vsctl("del-port %s %s" % (self.ovs_tunnel_bridge, tun_name))
                raise DebugException
                return

            # Give the system a moment to allocate the port number
            time.sleep(0.5)

        self.tunnels[tunnel] = (port_no, tun_name)

        ovs_ofctl("add-flow %s hard_timeout=0,idle_timeout=0,"\
                  "priority=1,in_port=%s,"
                  "actions=resubmit(,3)"
                  % (self.ovs_tunnel_bridge, port_no))

        cmd = "dump-flows %s" % self.ovs_tunnel_bridge
        flows = ovs_ofctl(cmd).splitlines()

        ports = []
        port_str = port_no 

        for x in flows:
            if "set_tunnel" in x:
                ports.append(x)
        
        for x in ports:
            port = x[x.find('output:') + len('output:'):]
            if len(port):
                port_str += ","
                port_str += port

        ovs_ofctl("mod-flows %s"
                  " hard_timeout=0,idle_timeout=0,priority=1,table=%s,"
                  "dl_vlan=%s,actions=strip_vlan,set_tunnel:%s,output:%s" 
                  % (self.ovs_tunnel_bridge, "21", "1", self.tunnel_key, port_str))

    def add_tunnel_broadcom(self, tunnel):
        global tun_id
        global open
        vlog.info("adding tunnel %s" % tunnel)
        encap, ip = tunnel.split("/")

        if encap != "vxlan_over_ipv4":
            vlog.warn("unsupported tunnel format %s" % encap)
            return

        tun_id += 1
        tun_name = "vx" + str(tun_id)

        try:
            x = tunnels[tunnel]
        except:
            tun_handle = open.create_tunnel(self.tunnel_key, self.short_name, tun_name, self.tunnel_key, ip);
            self.tunnels[tunnel] = (tun_name, tun_handle)

    def del_all_tunnels(self):
        for x in self.tunnels.keys():
            self.del_tunnel(x)

    def del_tenant(self):
        global open
        if open == None:
          open = VTEPOpEN()
        open.delete_tenant(self.tunnel_key)

    def del_tunnel(self, tunnel):
        if isBroadcomSwitch:
            self.del_tunnel_broadcom(tunnel)
        else:
            self.del_tunnel_ovs(tunnel)

    def del_tunnel_ovs(self, tunnel):
        vlog.info("removing tunnel %s" % tunnel)

        port_no, tun_name = self.tunnels[tunnel]
        ovs_ofctl("del-flows %s table=0,in_port=%s"
                    % (self.short_name, port_no))
        ovs_vsctl("del-port %s %s" % (self.short_name, tun_name))

        del self.tunnels[tunnel]

    def del_tunnel_broadcom(self, tunnel):
        global open

        vlog.info("removing tunnel %s" % tunnel)

        tun_name, tun_handle = self.tunnels[tunnel]
        if open == None:
          open = VTEPOpEN()
        open.delete_tunnel(tun_handle)

        del self.tunnels[tunnel]

    def update_local_macs(self, ps_name):
        if isBroadcomSwitch:
            self.update_local_macs_broadcom(ps_name)
        else:
            self.update_local_macs_ovs()

    '''
    broadcom-specific function to inject mac addr for a device into
    tenant tunnels
    '''

    def update_port_to_mac_bindings(self):
        return
        for interface in self.ports.iterkeys():
            if not interface.endswith("-l"):
                continue

            try:
                vlan, pp_name = interface.split("-")
            except:
                vlan, pp_name, pp_name1 = interface.split("-")

            mac = vtep_ctl("get physical_port %s description"
                            % (pp_name))
            if not mac:
                continue

            for x in self.tunnels.keys():
                open.add_forwarding_entry(self.tunnel_key, 
                                          mac, 
                                          self.tunnels[x][1])

    def update_local_macs_ovs(self): 
        flows = ovs_ofctl("dump-flows %s cookie=0x5000/-1,table=1"
                          % self.short_name).splitlines()
        macs = set()
        for f in flows:
            mac = re.split(r'.*dl_dst=(.*) .*', f)
            if len(mac) == 3:
                macs.add(mac[1])

        for mac in macs.difference(self.local_macs):
            vlog.info("adding local ucast %s to %s" % (mac, self.name))
            vtep_ctl("add-ucast-local %s %s %s" % (self.name, mac, Tunnel_Ip))

        for mac in self.local_macs.difference(macs):
            vlog.info("removing local ucast %s from %s" % (mac, self.name))
            vtep_ctl("del-ucast-local %s %s" % (self.name, mac))

        self.local_macs = macs

    def update_local_macs_broadcom(self, ps_name): 
        global open

        macs = open.get_macs(ps_name, self.name)

        for mac in macs.difference(self.local_macs):
            vlog.info("adding local ucast %s to %s" % (mac, self.name))
            vtep_ctl("add-ucast-local %s %s %s" % (self.name, mac, Tunnel_Ip))

        for mac in self.local_macs.difference(macs):
            vlog.info("removing local ucast %s from %s" % (mac, self.name))
            vtep_ctl("del-ucast-local %s %s" % (self.name, mac))

        self.local_macs = macs

    def add_remote_mac(self, mac, tunnel):
        if isBroadcomSwitch:
            return

        port_no = self.tunnels.get(tunnel, (0,""))[0]
        if not port_no:
            return

        ovs_ofctl("add-flow %s table=1,priority=1000,dl_dst=%s,action=%s"
                  % (self.short_name, mac, port_no))

    def del_remote_mac(self, mac):
        if isBroadcomSwitch:
            return

        ovs_ofctl("del-flows %s table=1,dl_dst=%s" % (self.short_name, mac))

    def update_remote_macs(self):
        vlog.info("update_remote_macs enter")

        remote_macs = {}
        unknown_dsts = set()
        tunnels = set()
        parse_ucast = True

        mac_list = vtep_ctl("list-remote-macs %s" % self.name).splitlines()
        vlog.info(mac_list)
        for line in mac_list:
            if (line.find("mcast-mac-remote") != -1):
                parse_ucast = False
                continue

            entry = re.split(r'  (.*) -> (.*)', line)
            if len(entry) != 4:
                continue

            if parse_ucast:
                remote_macs[entry[1]] = entry[2]
            else:
                if entry[1] != "unknown-dst":
                    continue

                unknown_dsts.add(entry[2])

            tunnels.add(entry[2])

        old_tunnels = set(self.tunnels.keys())

        for tunnel in tunnels.difference(old_tunnels):
            vlog.info("add tunnel")
            self.add_tunnel(tunnel)

        for tunnel in old_tunnels.difference(tunnels):
            vlog.info("del tunnel")
            self.del_tunnel(tunnel)

        for mac in remote_macs.keys():
            if (self.remote_macs.get(mac) != remote_macs[mac]):
                self.add_remote_mac(mac, remote_macs[mac])

        for mac in self.remote_macs.keys():
            if not remote_macs.has_key(mac):
                self.del_remote_mac(mac)

        self.remote_macs = remote_macs

        if (self.unknown_dsts != unknown_dsts):
            self.unknown_dsts = unknown_dsts
            self.update_flood()

        vlog.info("update_remote_macs leave")

    def update_stats(self):
        if isBroadcomSwitch:
            return

        # Map Open_vSwitch's "interface:statistics" to columns of
        # vtep's logical_binding_stats. Since we are using the 'interface'
        # from
        # the logical switch to collect stats, packets transmitted from it
        # is received in the physical switch and vice versa.
        stats_map = {'tx_packets':'packets_to_local',
                     'tx_bytes':'bytes_to_local',
                     'rx_packets':'packets_from_local',
                     'rx_bytes':'bytes_from_local'}

        # Go through all the logical switch's interfaces that end with "-l"
        # and copy the statistics to logical_binding_stats.
        for interface in self.ports.iterkeys():
            if not interface.endswith("-l"):
                continue

            vlog.info(interface)
            try:
                vlan, pp_name = interface.split("-")
            except:
                vlan, pp_name0, pp_name1, pp_name2 = interface.split("-")
                pp_name = pp_name0 + "-" + pp_name1 

            uuid = vtep_ctl("get physical_port %s vlan_stats:%s"
                            % (pp_name, vlan))
            if not uuid:
                continue

            for (mapfrom, mapto) in stats_map.iteritems():
                value = ovs_vsctl("get interface %s statistics:%s"
                                % (interface, mapfrom)).strip('"')
                vtep_ctl("set logical_binding_stats %s %s=%s"
                        % (uuid, mapto, value))

    def run(self, ps_name):
        if isBroadcomSwitch:
            self.update_port_to_mac_bindings()
        self.update_local_macs(ps_name)
        self.update_remote_macs()
        self.update_stats()

def add_binding(ps_name, binding, ls):
    if isBroadcomSwitch:
        add_binding_broadcom(ps_name, binding, ls)
    else:
        add_binding_ovs(ps_name, binding, ls)

def add_binding_ovs(ps_name, binding, ls):
    vlog.info("adding binding %s" % binding)

    try:
        vlan, pp_name = binding.split("-")
    except:
        vlan, pp_name0, pp_name1 = binding.split("-")
        pp_name = pp_name0 + pp_name1

    pbinding = binding+"-p"
    lbinding = binding+"-l"

    ls.add_lbinding(lbinding)
    Bindings[binding] = ls.name

def add_binding_broadcom(ps_name, binding, ls):

    vlog.info("adding binding %s" % binding)

    try:
        vlan, pp_name = binding.split("-")
    except:
        vlan, pp_name0, pp_name1 = binding.split("-")
        pp_name = pp_name0 + pp_name1

    pbinding = binding+"-p"
    lbinding = binding+"-l"

    vlan_ = vlan.lstrip('0')

    # Create a logical_bindings_stats record.
    if not vlan_:
        vlan_ = "0"
    vtep_ctl("set physical_port %s vlan_stats:%s=@stats --\
            --id=@stats create logical_binding_stats packets_from_local=0"\
            % (pp_name, vlan_))

    ls.add_lbinding(lbinding)
    Bindings[binding] = ls.name

def del_binding(ps_name, binding, ls):
    if isBroadcomSwitch:
        del_binding_broadcom(ps_name, binding, ls)
    else:
        del_binding_ovs(ps_name, binding, ls)

def del_binding_ovs(ps_name, binding, ls):
    vlog.info("removing binding %s" % binding)

    vlan, pp_name = binding.split("-")
    pbinding = binding+"-p"
    lbinding = binding+"-l"

    port_no = ovs_vsctl("get Interface %s ofport" % pp_name)
    patch_no = ovs_vsctl("get Interface %s ofport" % pbinding)
    vlan_ = vlan.lstrip('0')
    if vlan_:
        ovs_ofctl("del-flows %s in_port=%s,dl_vlan=%s"
                  % (ps_name, port_no, vlan_))
        ovs_ofctl("del-flows %s in_port=%s" % (ps_name, patch_no))
    else:
        ovs_ofctl("del-flows %s in_port=%s" % (ps_name, port_no))
        ovs_ofctl("del-flows %s in_port=%s" % (ps_name, patch_no))

    ls.del_lbinding(lbinding)

    # Destroy the patch port that connects the VLAN+port to the lswitch
    ovs_vsctl("del-port %s %s -- del-port %s %s"
              % (ps_name, pbinding, ls.short_name, lbinding))

    # Remove the record that links vlan with stats in logical_binding_stats.
    vtep_ctl("remove physical_port %s vlan_stats %s" % (pp_name, vlan))

    del Bindings[binding]

def del_binding_broadcom(ps_name, binding, ls):
    vlog.info("removing binding %s" % binding)

    vlan, pp_name = binding.split("-")
    pbinding = binding+"-p"
    lbinding = binding+"-l"

    port_no = ovs_vsctl("get Interface %s ofport" % pp_name)
    patch_no = ovs_vsctl("get Interface %s ofport" % pbinding)
    vlan_ = vlan.lstrip('0')
    if vlan_:
        ovs_ofctl("del-flows %s in_port=%s,dl_vlan=%s"
                  % (ps_name, port_no, vlan_))
        ovs_ofctl("del-flows %s in_port=%s" % (ps_name, patch_no))
    else:
        ovs_ofctl("del-flows %s in_port=%s" % (ps_name, port_no))
        ovs_ofctl("del-flows %s in_port=%s" % (ps_name, patch_no))

    ls.del_lbinding(lbinding)

    # Destroy the patch port that connects the VLAN+port to the lswitch
    ovs_vsctl("del-port %s %s -- del-port %s %s"
              % (ps_name, pbinding, ls.short_name, lbinding))

    # Remove the record that links vlan with stats in logical_binding_stats.
    vtep_ctl("remove physical_port %s vlan_stats %s" % (pp_name, vlan))

    del Bindings[binding]

def handle_physical(ps_name):
    # Gather physical ports except the patch ports we created

    if isBroadcomSwitch:
        handle_physical_broadcom(ps_name)
    else:
        handle_physical_ovs(ps_name)

def handle_physical_broadcom(ps_name):
    global exiting
    ovs_ports = open.get_interfaces()

    ovs_port_set = set([port for port in ovs_ports if port[-2:] != "-p"])

    vtep_pp_set = set(vtep_ctl("list-ports %s" % ps_name).split())

    for pp_name in ovs_port_set.difference(vtep_pp_set):
        if exiting == True:
          break
        vlog.info("adding %s to %s" % (pp_name, ps_name))
        vtep_ctl("add-port %s %s" % (ps_name, pp_name))

    for pp_name in vtep_pp_set.difference(ovs_port_set):
        if exiting == True:
          break
        vlog.info("deleting %s from %s" % (pp_name, ps_name))
        vtep_ctl("del-port %s %s" % (ps_name, pp_name))

    new_bindings = set()
    for pp_name in vtep_pp_set:
        if exiting == True:
          break
        binding_set = set(vtep_ctl("list-bindings %s %s"
                                   % (ps_name, pp_name)).splitlines())

        for b in binding_set:
            if exiting == True:
              break
            vlan, ls_name = b.split()
            if ls_name not in Lswitches:
                Lswitches[ls_name] = Logical_Switch(ls_name)

            binding = "%s-%s" % (vlan, pp_name)
            ls = Lswitches[ls_name]
            new_bindings.add(binding)

            if Bindings.has_key(binding):
                if Bindings[binding] == ls_name:
                    continue
                else:
                    del_binding(ps_name, binding,
Lswitches[Bindings[binding]])

            add_binding(ps_name, binding, ls)


    dead_bindings = set(Bindings.keys()).difference(new_bindings)
    for binding in dead_bindings:
        if exiting == True:
          break
        ls_name = Bindings[binding]
        ls = Lswitches[ls_name]

        del_binding(ps_name, binding, ls)

        if not len(ls.ports):
            del Lswitches[ls_name]

def handle_physical_ovs(ps_name):
    return handle_physical_ovs_broadcom(ps_name)

    # Gather physical ports except the patch ports we created

    ovs_ports = ovs_vsctl("list-ports %s" % ps_name).split()

    ovs_port_set = set([port for port in ovs_ports if port[-2:] != "-p"])

    vtep_pp_set = set(vtep_ctl("list-ports %s" % ps_name).split())

    for pp_name in ovs_port_set.difference(vtep_pp_set):
        vlog.info("adding %s to %s" % (pp_name, ps_name))
        vtep_ctl("add-port %s %s" % (ps_name, pp_name))

    for pp_name in vtep_pp_set.difference(ovs_port_set):
        vlog.info("deleting %s from %s" % (pp_name, ps_name))
        vtep_ctl("del-port %s %s" % (ps_name, pp_name))

    new_bindings = set()
    for pp_name in vtep_pp_set:
        binding_set = set(vtep_ctl("list-bindings %s %s"
                                   % (ps_name, pp_name)).splitlines())

        for b in binding_set:
            vlan, ls_name = b.split()
            if ls_name not in Lswitches:
                Lswitches[ls_name] = Logical_Switch(ls_name)

            binding = "%s-%s" % (vlan, pp_name)
            ls = Lswitches[ls_name]
            new_bindings.add(binding)

            if Bindings.has_key(binding):
                if Bindings[binding] == ls_name:
                    continue
                else:
                    del_binding(ps_name, binding,
Lswitches[Bindings[binding]])

            add_binding(ps_name, binding, ls)

    dead_bindings = set(Bindings.keys()).difference(new_bindings)
    for binding in dead_bindings:
        ls_name = Bindings[binding]
        ls = Lswitches[ls_name]

        del_binding(ps_name, binding, ls)

        if not len(ls.ports):
            ovs_vsctl("del-br %s" % Lswitches[ls_name].short_name)
            del Lswitches[ls_name]

def handle_physical_ovs_broadcom(ps_name):
    ls = vtep_ctl("list-ls").split('\n')
    for x in ls:
        if (x == None or x == "" or x in Lswitches):
            continue
        Lswitches[x] = Logical_Switch(x)

        ls = Lswitches[x]

def setup(ps_name):
    vtep_ctl("--all destroy Ucast_Macs_Local")
    vtep_ctl("--all destroy Ucast_Macs_Remote")
    vtep_ctl("--all destroy Mcast_Macs_Local")
    vtep_ctl("--all destroy Mcast_Macs_Remote")
    vtep_ctl("--all destroy Logical_Switch")
    vtep_ctl("--all destroy Physical_Locator_Set")
    vtep_ctl("--all destroy Physical_Switch")
    vtep_ctl("--all destroy Global")
    if isBroadcomSwitch:
        setup_broadcom(ps_name)
    else:
        setup_ovs(ps_name)

def setup_broadcom(ps_name):
    call_prog("vtep-ctl", ["--may-exist", "--", "add-ps", ps_name])

    call_prog("vtep-ctl", ["set", "physical_switch", ps_name,
                           'description="Broadcom Gateway"'])

    vtep_ctl("set physical_switch %s tunnel_ips='%s'"
              % (ps_name, vtepd_config.host_ip)).strip('[]\'"').split(", ")
    tunnel_ips = vtep_ctl("get physical_switch %s tunnel_ips"
                          % ps_name).strip('[]\'"').split(", ")
    if len(tunnel_ips) != 1 or not tunnel_ips[0]:
        ovs.util.ovs_fatal(0, "exactly one 'tunnel_ips' should be set", vlog)

    global Tunnel_Ip
    Tunnel_Ip = tunnel_ips[0]

def setup_ovs(ps_name):
    br_list = []

    call_prog("vtep-ctl", ["--may-exist", "--", "add-ps", ps_name])

    call_prog("vtep-ctl", ["set", "physical_switch", ps_name,
                           'description="Broadcom Gateway"'])

    vtep_ctl("set physical_switch %s tunnel_ips=%s"
              % (ps_name, get_ip_address(vtepd_config.local_interface))).strip('[]"').split(", ")
    tunnel_ips = vtep_ctl("get physical_switch %s tunnel_ips"
                          % ps_name).strip('[]"').split(", ")
    if len(tunnel_ips) != 1 or not tunnel_ips[0]:
        ovs.util.ovs_fatal(0, "exactly one 'tunnel_ips' should be set", vlog)

    global Tunnel_Ip
    Tunnel_Ip = tunnel_ips[0]

    ovs_ofctl("del-flows %s" % ps_name)

    # Remove any logical bridges from the previous run
    for br in br_list:
        if ovs_vsctl("br-get-external-id %s vtep_logical_switch"
                     % br) == "true":
            # Remove the remote side of any logical switch
            ovs_ports = ovs_vsctl("list-ports %s" % br).split()
            for port in ovs_ports:
                port_type = ovs_vsctl("get Interface %s type"
                                      % port).strip('"')
                if port_type != "patch":
                    continue

                peer = ovs_vsctl("get Interface %s options:peer"
                                 % port).strip('"')
                if (peer):
                    ovs_vsctl("del-port %s" % peer)

            ovs_vsctl("del-br %s" % br)

def build_mcast_macs_local():
    if isBroadcomSwitch:
        eth = "dtl0"
    else:
        eth = vtepd_config.local_interface 
 
    ls = vtep_ctl("list-ls").split('\n')
    for x in ls:
        if x != None and x != "":
            vtep_ctl("add-mcast-local %s unknown-dst vxlan_over_ipv4 %s" % (x,
get_ip_address(eth)))

class VTEPOpEN:

    '''
    Class to abstract the interface to OpEN overlay API.
    '''

    def __init__(self):
        self.open = OpENUtil()
        ret = self.open.connect("vtepd")
        if ret != OpEN.OPEN_E_NONE and ret != OpEN.OPEN_E_EXISTS:
            value = self.openapiErrorStrGet(ret)
            vlog.info("VTEPOpEN: unable to connect to open, rc %s" % (value))
            return
        self.clientHandle = self.open.get_client()
        self.macAddr_len = 6
        ret = self.enableVXLAN()
        if ret == OpEN.OPEN_E_NONE:
          err = True
          caps = self.getTunnelCaps()
          try:
            if caps != None and (caps & OpEN.OPEN_L2OL3_TUNNEL_TYPE_VXLAN) != OpEN.OPEN_L2OL3_TUNNEL_TYPE_VXLAN:
              pass
            else:
              err = False
          except:
              pass
        else:
          err = True
        if err == True:
            vlog.info("VTEPOpEN: overlay API (or VXLAN tunnels) not supported on this device")

    def __del__(self):
        pass

    def enableVXLAN(self):
      ret = OpEN.openapiL2oL3SwitchTunnelModeSet(self.clientHandle, OpEN.OPEN_L2OL3_TUNNEL_TYPE_VXLAN, OpEN.OPEN_ENABLE)
      return ret
      
    def getTunnelCaps(self):
        caps = None
        try:
          caps_p = OpEN.new_uint32_tp()
          ret = OpEN.openapiL2oL3TunnelCapabilitiesGet(self.clientHandle, caps_p)
          if ret == OpEN.OPEN_E_NONE:
            caps = OpEN.uint32_tp_value(caps_p)
        except:
            # no such API
            pass
        OpEN.delete_uint32_tp(caps_p)
        return caps

    def openapiErrorStrGet(self, error):

        '''
        Convert an OpEN error number into a string
        '''
        
        name = ""
        if error == OpEN.OPEN_E_NONE:
            name = "OPEN_E_NONE"
        elif error == OpEN.OPEN_E_RPC:
            name = "OPEN_E_RPC"
        elif error == OpEN.OPEN_E_INTERNAL:
            name = "OPEN_E_INTERNAL"
        elif error == OpEN.OPEN_E_PARAM:
            name = "OPEN_E_PARAM"
        elif error == OpEN.OPEN_E_FULL:
            name = "OPEN_E_FULL"
        elif error == OpEN.OPEN_E_EXISTS:
            name = "OPEN_E_EXISTS"
        elif error == OpEN.OPEN_E_TIMEOUT:
            name = "OPEN_E_TIMEOUT"
        elif error == OpEN.OPEN_E_FAIL:
            name = "OPEN_E_FAIL"
        elif error == OpEN.OPEN_E_DISABLED:
            name = "OPEN_E_DISABLED"
        elif error == OpEN.OPEN_E_UNAVAIL:
            name = "OPEN_E_UNAVAIL"
        elif error == OpEN.OPEN_E_NOT_FOUND:
            name = "OPEN_E_NOT_FOUND"
        elif error == OpEN.OPEN_E_EMPTY:
            name = "OPEN_E_EMPTY"
        elif error == OpEN.OPEN_E_ERROR:
            name = "OPEN_E_ERROR"
        else:
            name = "Unknown or Undefined Error"
        return name;

    def inet_pton(self, ip):
        return struct.unpack('I', socket.inet_pton(socket.AF_INET, ip))[0]

    def convertMacAddressStringIntoByte(self, mac):
        iNumber = ''
        ret = []

        mac = mac.split('-')

        for i in range(self.macAddr_len):
            iNumber = iNumber + str(mac[i])

        ret = ''.join(chr(int(iNumber[i:i+2], 16)) for i in range(0, len(iNumber), 2))
        return ret

    def get_interfaces(self):

        '''
        Retrieve a list of interfaces on the switch.
        '''

        ifs = []
         
        ifNum_p = OpEN.new_uint32_tp()
        ret = OpEN.openapiIfFirstGet(self.clientHandle,
                                     OpEN.OPEN_INTF_TYPE_PHY, ifNum_p)
        while ret == OpEN.OPEN_E_NONE:
            ifNum = OpEN.uint32_tp_value(ifNum_p)
            ifs.append(str(ifNum))                
            ret = OpEN.openapiIfNextGet(self.clientHandle, 
                                        OpEN.OPEN_INTF_TYPE_PHY, 
                                        ifNum, 
                                        ifNum_p)
        OpEN.delete_uint32_tp(ifNum_p)
        return ifs

    def create_tunnel(self, tenant, short_name, tun_name, tunnel_key, ip):
        '''
        For the given tenant, terminating IP, and tunnel key, create a 
        tunnel from the switch to that IP.
        '''

        handle = None

        handle_p = OpEN.new_uint32_tp()

        configType = OpEN.OPEN_L2OL3_CONFIG_CREATE

        ip = self.inet_pton(ip)
        ip = socket.ntohl(ip)

        retVal = OpEN.openapiL2oL3TunnelCreate(self.clientHandle, 
                                     configType,
                                     int(tenant),
                                     ip, 
                                     handle_p)

        if retVal == OpEN.OPEN_E_NONE:
            handle = OpEN.uint32_tp_value(handle_p)
            vlog.info("create_tunnel success. handle: %d" % handle)
        else:
            handle = OpEN.uint32_tp_value(handle_p)
            value = self.openapiErrorStrGet(retVal)
            vlog.info("create_tunnel failure: %s" % value)

        OpEN.delete_uint32_tp(handle_p)
        return handle
    
    def add_forwarding_entry(self, tunnel, mac, port):

        '''
        Add to the specified tunnel a forwarding entry for the
        given mac address and port.
        '''
 
        mac = mac.replace('"', '')
        macByte = self.convertMacAddressStringIntoByte(mac)
        mac_string_len = 6
        mac_string = self.open.getCharBuffer(mac_string_len, macByte)
        mac_buf = OpEN.open_buffdesc();
        mac_buf.pstart = mac_string
        mac_buf.size = mac_string_len
        retVal = OpEN.openapiL2oL3TenantL2FwdEntryAdd(self.clientHandle,
                                                      int(tunnel),
                                                      mac_buf,
                                                      int(port))
        if retVal != OpEN.OPEN_E_NONE:
            value = self.openapiErrorStrGet(retVal)
            vlog.info("add_forwarding_entry failed, rc %s" % (value))

    def delete_tunnel(self, tunnelHandle):
        '''
        delete the specified tunnel
        '''
        retVal = OpEN.openapiL2oL3TunnelDelete(self.clientHandle, tunnelHandle)
        if retVal != OpEN.OPEN_E_NONE:
            value = self.openapiErrorStrGet(retVal)
            vlog.info("Tunnel deletion failed, rc %s" % (value))

    def get_macs(self, ps_name, ls_name):
        '''
        Currently, ICOS OpEN does not support access to the FDB.
        So, what we do is look into the data being written to the
        Physical_Port table to gather the mac addr of devices that
        are configured for this tenant/logical_switch. Once we have
        FDB access in OpEN, we can do something else perhaps. It
        remains to be seen if we would want to know anything beyond
        what was configured as devices, however.
        '''

        macs = set()
        
        # list ports

        ports = set(vtep_ctl("list-ports %s" % ps_name).split())

        # for each port, get bindings, for which there should be
        # only one

        for port in ports:
            bindings = set(vtep_ctl("list-bindings %s %s"
                           % (ps_name, port)).splitlines())
            for binding in bindings:
                ls = binding.split(' ')[1]
                if ls == ls_name:
                    # get the description field, which contains the mac addr
                    # of the device plugged into the port

                    mac = vtep_ctl("get physical_port %s description" % port)
                    mac = mac.replace('-', ':').strip('"')
                    macs.add(mac)

        return macs

    def create_tenant(self, tenant, tunnelType, vlanID):
        '''
        Create a tenant on the switch.
        '''

        configType = OpEN.OPEN_L2OL3_CONFIG_CREATE

        config = OpEN.open_l2ol3TenantConfig_t()
        config.localTepIpAddr = self.inet_pton(vtepd_config.host_ip)
        config.vlanId = int(vlanID)
        config.tunnelType = tunnelType
        tenant = int(tenant)

        config.localTepIpAddr = socket.ntohl(config.localTepIpAddr)

        retVal = OpEN.openapiL2oL3TenantCreate(self.clientHandle, configType, tenant, config)
        if retVal != OpEN.OPEN_E_NONE:
            value = self.openapiErrorStrGet(retVal)
            vlog.info("Tenant creation failed, rc = %s" % (value))
        else:
            vlog.info("Tenant creation success")

    def delete_tenant(self, tenant):
        '''
        Delete a tenant on the switch.
        '''

        tenant = int(tenant)
        retVal = OpEN.openapiL2oL3TenantDelete(self.clientHandle, tenant)

        if retVal != OpEN.OPEN_E_NONE:
            value = self.openapiErrorStrGet(retVal)
            vlog.info("Tenant deletion failed, rc = %s" % (value))
        else:
            vlog.info("Tenant deletion success")

    def create_vlan(self, vlan):
        '''
        Create a VLAN on the switch.
        '''

        vlan = int(vlan)
        result = OpEN.openapiVlanCreatedCheck(self.clientHandle, vlan)
        if result != OpEN.OPEN_E_NONE:
            result = OpEN.openapiVlanCreate(self.clientHandle, vlan)
            if result == OpEN.OPEN_E_FAIL:
                value = self.openapiErrorStrGet(result)
                vlog.info("failed to create vlan: %d error %s" % (vlan, value))
            else:
                vlog.info("success create vlan: %d" % (vlan))

    def delete_vlan(self, vlan):
        '''
        Delete a VLAN from the switch
        '''

        vlan = int(vlan)
        result = OpEN.openapiVlanDelete(self.clientHandle, vlan)
        if result == OpEN.OPEN_E_NONE:
            value = self.openapiErrorStrGet(result)
            vlog.info("failed to delete vlan %d error %s" % (vlan, value))
        else:
            vlog.info("success delete vlan %d" % (vlan))

    def add_port_to_vlan(self, vlan, port):
        '''
        Add the specified port to the specified VLAN
        '''

        vlan = int(vlan)
        port = int(port)
        result = OpEN.openapiVlanIfParticipationSet(self.clientHandle,
                                                    vlan,
                                                    port,
                                                    OpEN.OPEN_VLAN_PARTICIPATION_MODE_INCLUDE)

        if result != OpEN.OPEN_E_NONE:
            value = self.openapiErrorStrGet(result)
            vlog.info("failed to add port %d to vlan %d error %s" % (port, vlan, error))
        else:
            vlog.info("success add port %d to vlan %d" % (port, vlan))

    def remove_port_from_vlan(self, vlan, port):
        '''
        Remove the specified port to the specified VLAN
        '''

        vlan = int(vlan)
        port = int(port)
        result = OpEN.openapiVlanIfParticipationSet(self.clientHandle,
                                                    vlan,
                                                    port,
                                                    OpEN.OPEN_VLAN_PARTICIPATION_MODE_EXCLUDE)

        if result != OpEN.OPEN_E_NONE:
            value = self.openapiErrorStrGet(result)
            vlog.info("failed to remove port %d to vlan %d error %s" % (port, vlan, value))
        else:
            vlog.info("success remove port %d to vlan %d" % (port, vlan))

    def set_port_pvid(self, port, pvid):
        '''
        Set the PVID on a port to the specified value
        '''
        port = int(port)
        pvid = int(pvid)
        result = OpEN.openapiVlanIfPvidSet(self.clientHandle,
                                           port,
                                           pvid)
        if result != OpEN.OPEN_E_NONE:
            value = self.openapiErrorStrGet(result)
            vlog.info("failed to set pvid %d on port %d error %s" % (pvid, port, value))
        else:
            vlog.info("success set pvid %d on port %d" % (pvid, port))

    def set_port_ip(self, port, ip, prefixLen):
        '''
        Set the port IP address to the given value 
        '''

        port = int(port)
        ipAddr = OpEN.OPEN_INTF_IP_ADDR_TYPE_t()
        ipAddr.ipAddr.family = OpEN_AF_INET
        ipAddr.ipAddr.addr.ipv4 = self.inet_pton(ip)
        ipAddr.pfxLen = prefixLen
        
        ipAddr.ipAddr.addr.ipv4 = socket.ntohl(ipAddr.ipAddr.addr.ipv4)

        result = OpEN.openapiVlanIfPvidSet(self.clientHandle,
                                       OpEN.OPEN_AF_INET,
                                       port,
                                       OpEN.OPEN_IP_ADDR_TYPE_STATIC_PRIMARY,
                                       ipAddr,
                                       OpEN.OPEN_ENABLE)
        if result != OpEN.OPEN_E_NONE:
            value = self.openapiErrorStrGet(result)
            vlog.info("failed to set ip on port %d error %s" % (port, value))
        else:
            vlog.info("success set ip on port %d" % (port))

def cleanup():
    global Lswitches
    global Bindings
    vlog.info("cleanup: enter")
    for x in Lswitches:
        Lswitches[x].cleanup()
        del x
    for x in Bindings:
        pass

def main():
    global exiting
    parser = argparse.ArgumentParser()
    parser.add_argument("ps_name", metavar="PS-NAME",
                        help="Name of physical switch.")
    parser.add_argument("--root-prefix", metavar="DIR",
                        help="Use DIR as alternate root directory"
                        " (for testing).")
    parser.add_argument("--version", action="version",
                        version="%s %s" % (ovs.util.PROGRAM_NAME, VERSION))

    ovs.vlog.add_args(parser)
    ovs.daemon.add_args(parser)
    args = parser.parse_args()
    ovs.vlog.handle_args(args)
    ovs.daemon.handle_args(args)

    global root_prefix
    if args.root_prefix:
        root_prefix = args.root_prefix

    ps_name = args.ps_name

    ovs.daemon.daemonize()

    ovs.unixctl.command_register("exit", "", 0, 0, unixctl_exit, None)
    error, unixctl = ovs.unixctl.server.UnixctlServer.create(None,
                                                             version=VERSION)
    if error:
        ovs.util.ovs_fatal(error, "could not create unixctl server", vlog)

    if isBroadcomSwitch:
        global open
        open = VTEPOpEN()

    setup(ps_name)

    signal.signal(signal.SIGHUP, receive_signal)

    try:
      while True: 
        unixctl.run()
        if exiting == True:
            vlog.info("main: exiting main loop")
            break
        handle_physical(ps_name)

        for ls_name, ls in Lswitches.items():
            ls.run(ps_name)

        build_mcast_macs_local()

        poller = ovs.poller.Poller()
        unixctl.wait(poller)
        poller.timer_wait(1000)
        poller.block()
    except:
      vlog.info("main: exception in main loop")

    cleanup()

    unixctl.close()

if __name__ == '__main__':

    try:
        main()
    except SystemExit:
        # Let system.exit() calls complete normally
        raise
    except:
        vlog.exception("traceback")
        sys.exit(ovs.daemon.RESTART_EXIT_CODE)

